#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu

def get_random_objects(model=None, queryset=None, count=float('+inf')):
    """
       Get `count` random objects for a model object `model` or from
       a queryset. Returns an iterator that yield one object at a time.

       You model must have an auto increment id for it to work and it should
       be available on the `id` attribute.
    """

    if not queryset:
        try:
            queryset = model.objects.all()
        except AttributeError:
            raise ValueError("You must provide a model or a queryset")

    max_ = queryset.aggregate(Max('id'))['id__max']
    i = 0
    while i < count:
      try:
          yield queryset.get(pk=randint(1, max_))
          i += 1
      except queryset.model.DoesNotExist:
          pass


from .names import names as default_names
import random

class NameGenerator(object):
    def __init__(self, names=default_names):
        self._names = {i : name.strip() for i, name in enumerate(names)}
        self._total_names = len(self._names)
        self._used_indices = set()
    def __call__(self):
        index = random.randrange(self._total_names)
        name = self._names[index]
        if index not in self._used_indices:
            self._used_indices.add(index)
            return name
    def __iter__(self):
        while True:
            yield self()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu

import re
import random

from django.conf import settings
from django.views.static import serve
from django.contrib.staticfiles.views import serve as serve_static
from django.shortcuts import redirect

from django.contrib.auth.models import User

from namegen.namegen import NameGenerator

from utils import setting


class ForceSuperUserMiddleWare(object):
    """
        Developpement middleware forcing login with a super user so you
        don't have to login or worry about access rights.
    """

    def process_request(self, request):

        request.user = User.objects.filter(is_superuser=True)[0]


class StaticServe(object):
    """
        Django middleware for serving static files instead of using urls.py.

        It serves them wether you are set DEBUG or not, so put it into
        a separate settings file to activate it at will.
    """

    media, static = settings.MEDIA_URL.rstrip('/'), settings.STATIC_URL.rstrip('/')
    media_regex = re.compile(r'^%s/(?P<path>.*)$' % media)
    static_regex = re.compile(r'^%s/(?P<path>.*)$' % static)


    def process_request(self, request):

        match = self.media_regex.search(request.path)
        if match:
            return serve(request, match.group(1), settings.MEDIA_ROOT)

        match = self.static_regex.search(request.path)
        if match:
            return serve_static(request, match.group(1), insecure=True)


class AutoLogNewUser(object):


    CALLBACK = setting('AUTOLOGNEWUSER_CALLBAK', None)


    def process_request(self, request):

        if not request.user.is_authenticated():

            user_id = request.session.get('django_quicky:user_id', None)

            if not user_id:
                first_name = iter(NameGenerator()).next().title()
                username = "%s%s" % (first_name, random.randint(10, 100))
                user = User.objects.create(username=username, first_name=first_name)
                request.session['django_quicky:user_id'] = user.pk
                res = request.path
                if self.CALLBACK:
                    res = self.CALLBACK(request)
                return redirect(res or request.path)

            else:
                request.user = User.objects.get(pk=user_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu

from django.http import HttpResponse
from django.conf import settings
from django.shortcuts import _get_queryset

__all__ = ['HttpResponseException', 'get_object_or_None', 'setting']


class HttpResponseException(HttpResponse, Exception):
    pass


def get_object_or_None(klass, *args, **kwargs):
    """
        Uses get() to return an object or None if the object does not exist.

        klass may be a Model, Manager, or QuerySet object. All other passed
        arguments and keyword arguments are used in the get() query.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def setting(name, default):
    """
        Gets settings from django.conf if exists, returns default value otherwise

        Example:

        DEBUG = setting('DEBUG', False)
    """
    return getattr(settings, name, default)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


from django.views.debug import get_safe_settings


class SafeSettings(object):
    """
        Map attributes to values in the safe settings dict
    """
    def __init__(self):
        self._settings = get_safe_settings()

    def __getattr__(self, name):
        try:
            return self._settings[name.upper()]
        except KeyError:
            raise AttributeError


settings_obj = SafeSettings()


def settings(request):
    return {'settings': settings_obj}

